# Mashroom Auth

The Auth service is intended to support user authentication and authorization based on [JWT](https://jwt.io/).

The service uses [Keycloak](https://www.keycloak.org/) as identity engine and shares the primary user ID with Account service. That ID value is contained in `sub` claim of the token.

## GRPC API

Method | Scope | Description
-|-|-
`Auth` | Internal | Returns authentication credentials for the provided JWT access token
`Login` | Public | Performs login using email/password
`OauthLogin` | Public | Performs OAuth login
`Refresh` | Public | Refreshes the provided access/refresh tokens, returning their re-issued versions
`Logout` | Public | Performs logout
`LogoutAll` | Public | Performs logout (all sessions)
`Signup` | Public | Creates new user account (via `Create`) and sends email notification
`Create` | Internal | Creates new user account
`Update` | Public | Updates user account
`ForgotPassword` | Public| Resets user password
`UpdatePin` | Public| Updates PIN number
`VerifyPhone` | Public| Verifies phone by OTP
`VerifyEmail` | Public| Verifies email by OTP
`BlockUser` | Internal | Blocks user account
`CheckPassword` | Internal | Checks if the password is correct by logging in and out
`ResetAllAuthAttempts` | Internal | Removes auth attempts data
`ServiceUpdatePassword` | Internal| Updates user password

(TODO: a more detailed description of the methods is needed)

## GRPC coupling

Service | Use
-|-
Account | User accounts store
Notification | Notification by email and sms
Invitation | Phone and email OTP (one time password) verification

## Databases

DBMS | Extensions
-|-
PostgreSQL | uuid-ossp
  
Access abstraction: "DAL/Repository pattern".  
Migrations: [embedded](https://github.com/golang-migrate/migrate), applied at service startup.  

## Zeebe integration

N/A

## Observability

N/A

## Notes

Keycloak access (via HTTP requests) is abstracted as database access. The data structures for this are generated from the Keycloak REST API documentation.
