package service

import (
	"context"
	"errors"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/common/api/grpcerr"
)

func (srv *serverImpl) accountLoad(acc *account.Account) (*account.Account, []auth.AuthErrors, error) {
	if acc.GetBlockValue() {
		return nil, []auth.AuthErrors{
			auth.AuthErrors_AUTH_ERROR_ACCOUNT_IS_BLOCKED,
		}, nil
	}

	return acc, nil, nil
}

func (srv *serverImpl) accountLoadEmail(
	ctx context.Context,
	email string,
	adminSite bool,
) (*account.Account, []auth.AuthErrors, error) {
	if adminSite {
		return nil, nil, nil
	}

	respAcc, err := srv.Connections.Account.GetByEmail(ctx, &account.GetByEmailRequest{Email: email})
	if errors.Is(err, grpcerr.NotFound) {
		return nil, []auth.AuthErrors{
			auth.AuthErrors_AUTH_ERROR_ACCOUNT_DOES_NOT_EXIST,
		}, nil
	}

	if err != nil {
		return nil, nil, err
	}

	return srv.accountLoad(respAcc.GetAccount())
}

func (srv *serverImpl) accountLoadID(
	ctx context.Context,
	userID string,
	adminSite bool,
) (*account.Account, []auth.AuthErrors, error) {
	respAcc, err := srv.Connections.Account.Get(ctx, &account.GetRequest{Id: userID, Admin: adminSite})
	if errors.Is(err, grpcerr.NotFound) {
		return nil, nil, errUserNotFound
	}

	if err != nil {
		return nil, nil, err
	}

	return srv.accountLoad(respAcc.GetAccount())
}
