// Package service provides implementation for auth service
package service

import (
	"fmt"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/auth/internal/model"
	"gitlab.mashroom.com/mashroom/backend/auth/internal/storage"
	invitation "gitlab.mashroom.com/mashroom/backend/invitation/api"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
)

// Service contract
type Service interface {
	auth.AuthServiceServer
}

// Connections remote service connections
type Connections struct {
	Account      account.AccountServiceClient
	Notification notification.NotificationServiceClient
	Invitation   invitation.InvitationServiceClient
}

// Config for Service
type Config struct {
	Repo          model.Repository
	Storage       storage.Repository
	Connections   Connections
	ReCaptchaKey  string
	AuthThreshold int
	OAuthEnabled  bool
}

func validateConfigConnections(connections *Connections) error {
	if connections.Account == nil {
		return fmt.Errorf("no account client")
	}

	return nil
}

func validateConfig(config *Config) error {
	if config == nil {
		return fmt.Errorf("invalid service config")
	}

	if err := validateConfigConnections(&config.Connections); err != nil {
		return err
	}

	if config.Repo == nil {
		return fmt.Errorf("no repository in service config")
	}

	return nil
}

// NewService returns new Service instance
func NewService(config *Config) (Service, error) {
	err := validateConfig(config)
	if err != nil {
		return nil, err
	}

	return &serverImpl{
		Config: *config,
	}, nil
}

type serverImpl struct {
	Config
}
