package test

import (
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/auth/internal/storage"
)

func (s *serviceSuite) TestAuthLogoutAll_ok_password() {
	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	s.storage.EXPECT().SessionLogin(s.testCtx, s.expectUserEmail(), "rrr").Return(s.expectSessionAuth(1), nil)
	s.storage.EXPECT().SessionRemove(s.testCtx, s.expectSessionState(1)).Return(nil)

	s.storage.EXPECT().SessionRemoveAll(s.testCtx, s.expectUserID()).Return(nil)

	resp, err := s.srv.LogoutAll(s.testCtx, &auth.LogoutAllRequest{
		AccessToken: s.expectAccessToken(0),
		Password:    &auth.LogoutAllRequest_PasswordValue{PasswordValue: "rrr"},
		Pin:         nil,
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)
}

func (s *serviceSuite) TestAuthLogoutAll_ok_pin() {
	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	s.storage.EXPECT().UserGetByID(s.testCtx, s.expectUserID()).Return(&storage.User{
		Access: nil,
		Attributes: map[string]interface{}{
			"pin": "ppp",
		},
		ClientConsents:             nil,
		ClientRoles:                nil,
		CreatedTimestamp:           0,
		Credentials:                nil,
		DisableableCredentialTypes: nil,
		Email:                      "",
		EmailVerified:              false,
		Enabled:                    false,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   "",
	}, nil)

	s.storage.EXPECT().SessionRemoveAll(s.testCtx, s.expectUserID()).Return(nil)

	resp, err := s.srv.LogoutAll(s.testCtx, &auth.LogoutAllRequest{
		AccessToken: s.expectAccessToken(0),
		Password:    nil,
		Pin:         &auth.LogoutAllRequest_PinValue{PinValue: "ppp"},
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)
}

func (s *serviceSuite) TestAuthLogoutAll_error_verification() {
	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	resp, err := s.srv.LogoutAll(s.testCtx, &auth.LogoutAllRequest{
		AccessToken: s.expectAccessToken(0),
		Password:    nil,
		Pin:         nil,
	})

	s.Require().NoError(err)

	s.Require().Contains(resp.GetErrors(), auth.AuthErrors_AUTH_ERROR_VERIFICATION_REQUIRED)
}

func (s *serviceSuite) TestAuthLogoutAll_error_password() {
	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	s.storage.EXPECT().SessionLogin(s.testCtx, s.expectUserEmail(), "rrr").Return(nil, s.expectError())

	resp, err := s.srv.LogoutAll(s.testCtx, &auth.LogoutAllRequest{
		AccessToken: s.expectAccessToken(0),
		Password:    &auth.LogoutAllRequest_PasswordValue{PasswordValue: "rrr"},
		Pin:         nil,
	})

	s.Require().NoError(err)

	s.Require().Contains(resp.GetErrors(), auth.AuthErrors_AUTH_ERROR_VERIFICATION_INVALID)
}

func (s *serviceSuite) TestAuthLogoutAll_error_pin() {
	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	s.storage.EXPECT().UserGetByID(s.testCtx, s.expectUserID()).Return(&storage.User{
		Access: nil,
		Attributes: map[string]interface{}{
			"pin": "zzzz",
		},
		ClientConsents:             nil,
		ClientRoles:                nil,
		CreatedTimestamp:           0,
		Credentials:                nil,
		DisableableCredentialTypes: nil,
		Email:                      "",
		EmailVerified:              false,
		Enabled:                    false,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   "",
	}, nil)

	resp, err := s.srv.LogoutAll(s.testCtx, &auth.LogoutAllRequest{
		AccessToken: s.expectAccessToken(0),
		Password:    nil,
		Pin:         &auth.LogoutAllRequest_PinValue{PinValue: "ppp"},
	})

	s.Require().NoError(err)

	s.Require().Contains(resp.GetErrors(), auth.AuthErrors_AUTH_ERROR_VERIFICATION_INVALID)
}

func (s *serviceSuite) TestAuthLogoutAll_error_introspect() {
	s.storage.EXPECT().SessionIntrospect(s.testCtx, s.expectAccessToken(0)).Return(nil, s.expectError())

	resp, err := s.srv.LogoutAll(s.testCtx, &auth.LogoutAllRequest{
		AccessToken: s.expectAccessToken(0),
		Password:    nil,
		Pin:         nil,
	})

	s.Require().Error(err)

	s.Require().Nil(resp)
}
