package test

import (
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/auth/internal/storage"
)

func (s *serviceSuite) TestAuthOauthLogin_ok() {
	s.storage.EXPECT().UserGetByEmail(s.testCtx, s.expectUserEmail()).Return([]*storage.User{
		{
			Enabled: true,
			Id:      s.expectUserID(),
		},
	}, nil)

	s.repo.EXPECT().UserAttrGetByNameValue(s.testCtx, "oauth", "rrr").Return([]string{s.expectUserID()}, nil)

	s.storage.EXPECT().SessionImpersonate(s.testCtx, s.expectUserID()).Return(s.expectSessionAuth(0), nil)

	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	s.repo.EXPECT().AuthAttemptRemoveByRemote(s.testCtx, s.expectUserRemote()).Return(nil)

	s.account.EXPECT().UpdateLastLogin(s.testCtx, &account.UpdateLastLoginRequest{
		Id: s.expectUserID(),
	}).Return(&account.UpdateLastLoginResponse{}, nil)

	s.repo.EXPECT().RecordUserLogin(s.testCtx, s.expectUserID(), s.expectUserRemote()).Return(false, true, nil)

	resp, err := s.srv.OauthLogin(s.testCtx, &auth.OauthRequest{
		Email:       &auth.OauthRequest_EmailValue{EmailValue: s.expectUserEmail()},
		TosAccepted: nil,
		PhoneOtp:    nil,
		EmailOtp:    nil,
		Remote:      &auth.OauthRequest_RemoteValue{RemoteValue: s.expectUserRemote()},
		OauthId:     &auth.OauthRequest_OauthIdValue{OauthIdValue: "rrr"},
		OauthType:   nil,
		Captcha:     nil,
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)
}

func (s *serviceSuite) TestAuthOauthLogin_error_update_last_login() {
	s.storage.EXPECT().UserGetByEmail(s.testCtx, s.expectUserEmail()).Return([]*storage.User{
		{
			Enabled: true,
			Id:      s.expectUserID(),
		},
	}, nil)

	s.repo.EXPECT().UserAttrGetByNameValue(s.testCtx, "oauth", "rrr").Return([]string{s.expectUserID()}, nil)

	s.storage.EXPECT().SessionImpersonate(s.testCtx, s.expectUserID()).Return(s.expectSessionAuth(0), nil)

	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	s.repo.EXPECT().AuthAttemptRemoveByRemote(s.testCtx, s.expectUserRemote()).Return(nil)

	s.account.EXPECT().UpdateLastLogin(s.testCtx, &account.UpdateLastLoginRequest{
		Id: s.expectUserID(),
	}).Return(nil, s.expectError())

	s.repo.EXPECT().RecordUserLogin(s.testCtx, s.expectUserID(), s.expectUserRemote()).
		Return(false, false, s.expectError())

	_, err := s.srv.OauthLogin(s.testCtx, &auth.OauthRequest{
		Email:       &auth.OauthRequest_EmailValue{EmailValue: s.expectUserEmail()},
		TosAccepted: nil,
		PhoneOtp:    nil,
		EmailOtp:    nil,
		Remote:      &auth.OauthRequest_RemoteValue{RemoteValue: s.expectUserRemote()},
		OauthId:     &auth.OauthRequest_OauthIdValue{OauthIdValue: "rrr"},
		OauthType:   nil,
		Captcha:     nil,
	})

	s.Require().Error(err)
}

func (s *serviceSuite) TestAuthOauthLogin_error_no_users() {
	s.storage.EXPECT().UserGetByEmail(s.testCtx, s.expectUserEmail()).Return(nil, nil)

	_, err := s.srv.OauthLogin(s.testCtx, &auth.OauthRequest{
		Email:       &auth.OauthRequest_EmailValue{EmailValue: s.expectUserEmail()},
		TosAccepted: nil,
		PhoneOtp:    nil,
		EmailOtp:    nil,
		Remote:      &auth.OauthRequest_RemoteValue{RemoteValue: s.expectUserRemote()},
		OauthId:     &auth.OauthRequest_OauthIdValue{OauthIdValue: "rrr"},
		OauthType:   nil,
		Captcha:     nil,
	})

	s.Require().Error(err)
}
