package test

import (
	"github.com/golang/mock/gomock"
	"gitlab.mashroom.com/mashroom/backend/auth/internal/storage"
	"gitlab.mashroom.com/mashroom/backend/auth/keycloak"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"

	invitation "gitlab.mashroom.com/mashroom/backend/invitation/api"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
)

func (s *serviceSuite) TestAuthSignup_ok_platform() {
	acc := &account.Account{
		Id:        nil,
		CreatedAt: nil,
		Details: &account.Account_DetailsValue{
			DetailsValue: &account.AccountDetails{
				FirstName: &account.AccountDetails_FirstNameValue{FirstNameValue: "aaa"},
				LastName:  &account.AccountDetails_LastNameValue{LastNameValue: "bbb"},
				Gender:    nil,
				UkNi:      nil,
				Birthday:  nil,
				Avatar:    nil,
			},
		},
		Location: nil,
		Contact: &account.Account_ContactValue{
			ContactValue: &account.AccountContact{
				EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
				PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
				PhonesSecondary: nil,
				ContactStatus:   nil,
			},
		},
		Favorites:           nil,
		Admin:               nil,
		Block:               nil,
		TosVersion:          nil,
		LastLogin:           nil,
		Deleted:             nil,
		Log:                 nil,
		Test:                nil,
		BankDetails:         nil,
		Resources:           nil,
		CreationType:        nil,
		ViewedVideosIds:     nil,
		EnableLandlordTool:  nil,
		Entrypoint:          0,
		AuthorizationOrigin: 0,
	}

	s.account.EXPECT().Verify(s.testCtx, &account.VerifyRequest{
		Account: acc,
	}).Return(&account.VerifyResponse{
		Valid:        true,
		Verification: nil,
	}, nil)

	s.storage.EXPECT().UserGetByEmail(s.testCtx, s.expectUserEmail()).Return(nil, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: "newuser-" + s.expectUserRemote(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
			Key:    "71002003040",
			UserId: "newuser-" + s.expectUserRemote(),
			Remote: "",
			Otp:    "123456",
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: "newuser-" + s.expectUserRemote(),
		Remote: "",
		Otp:    "123456",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	s.storage.EXPECT().UserCreate(s.testCtx, &storage.User{
		Access: map[string]interface{}{
			"manageGroupMembership": true,
			"view":                  true,
			"mapRoles":              true,
			"impersonate":           true,
			"manage":                true,
		},
		Attributes: map[string]interface{}{
			"oauth_type":        "",
			"oauth":             "",
			"email":             s.expectUserEmail(),
			"pin":               "",
			"transient_session": "trans1",
			"phoneNumber":       "71002003040",
		},
		ClientConsents:   nil,
		ClientRoles:      nil,
		CreatedTimestamp: 0,
		Credentials: []keycloak.CredentialRepresentation{
			{
				CreatedDate:    0,
				CredentialData: "",
				Id:             "",
				Priority:       0,
				SecretData:     "",
				Temporary:      false,
				Type:           "password",
				UserLabel:      "",
				Value:          "rrrrrrrr",
			},
		},
		DisableableCredentialTypes: []string{"password"},
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    true,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "aaa",
		Groups:                     nil,
		Id:                         "",
		LastName:                   "bbb",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}).Return(nil)

	s.storage.EXPECT().SessionImpersonate(s.testCtx, s.expectUserEmail()).Return(s.expectSessionAuth(0), nil)

	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	s.account.EXPECT().Create(s.testCtx, &account.CreateRequest{
		Account: acc,
	}).Return(&account.CreateResponse{
		Account:      acc,
		Success:      false,
		Valid:        true,
		Verification: nil,
	}, nil)

	s.notification.EXPECT().SendNotification(s.testCtx, gomock.Any()).Return(
		&notification.SendNotificationResponse{},
		nil,
	)

	resp, err := s.srv.Signup(s.testCtx, &auth.SignupRequest{
		Account:          acc,
		Password:         &auth.SignupRequest_PasswordValue{PasswordValue: "rrrrrrrr"},
		Pin:              nil,
		PhoneOtp:         &auth.SignupRequest_PhoneOtpValue{PhoneOtpValue: "123456"},
		EmailOtp:         nil,
		TosAccepted:      &auth.SignupRequest_TosAcceptedValue{TosAcceptedValue: 1},
		Remote:           &auth.SignupRequest_RemoteValue{RemoteValue: s.expectUserRemote()},
		OauthId:          nil,
		OauthType:        nil,
		AdminSite:        nil,
		TransientSession: &auth.SignupRequest_TransientSessionValue{TransientSessionValue: "trans1"},
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Empty(resp.GetErrors())
}

func (s *serviceSuite) TestAuthSignup_ok_admin() {
	acc := &account.Account{
		Id:        nil,
		CreatedAt: nil,
		Details: &account.Account_DetailsValue{
			DetailsValue: &account.AccountDetails{
				FirstName: &account.AccountDetails_FirstNameValue{FirstNameValue: "aaa"},
				LastName:  &account.AccountDetails_LastNameValue{LastNameValue: "bbb"},
				Gender:    nil,
				UkNi:      nil,
				Birthday:  nil,
				Avatar:    nil,
			},
		},
		Location: nil,
		Contact: &account.Account_ContactValue{
			ContactValue: &account.AccountContact{
				EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
				PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
				PhonesSecondary: nil,
				ContactStatus:   nil,
			},
		},
		Favorites:           nil,
		Admin:               &account.Account_AdminValue{AdminValue: true},
		Block:               nil,
		TosVersion:          nil,
		LastLogin:           nil,
		Deleted:             nil,
		Log:                 nil,
		Test:                nil,
		BankDetails:         nil,
		Resources:           nil,
		CreationType:        nil,
		ViewedVideosIds:     nil,
		EnableLandlordTool:  nil,
		Entrypoint:          0,
		AuthorizationOrigin: 0,
	}

	s.account.EXPECT().Verify(s.testCtx, &account.VerifyRequest{
		Account: acc,
	}).Return(&account.VerifyResponse{
		Valid:        true,
		Verification: nil,
	}, nil)

	s.storage.EXPECT().UserGetByEmail(s.testCtx, s.expectUserEmail()).Return(nil, nil)

	s.storage.EXPECT().UserCreate(s.testCtx, &storage.User{
		Access: map[string]interface{}{
			"manageGroupMembership": true,
			"view":                  true,
			"mapRoles":              true,
			"impersonate":           true,
			"manage":                true,
		},
		Attributes: map[string]interface{}{
			"oauth_type":        "",
			"oauth":             "",
			"email":             s.expectUserEmail(),
			"pin":               "",
			"transient_session": "trans1",
			"phoneNumber":       "71002003040",
		},
		ClientConsents:   nil,
		ClientRoles:      nil,
		CreatedTimestamp: 0,
		Credentials: []keycloak.CredentialRepresentation{
			{
				CreatedDate:    0,
				CredentialData: "",
				Id:             "",
				Priority:       0,
				SecretData:     "",
				Temporary:      false,
				Type:           "password",
				UserLabel:      "",
				Value:          "rrrrrrrr",
			},
		},
		DisableableCredentialTypes: []string{"password"},
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    true,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "aaa",
		Groups:                     nil,
		Id:                         "",
		LastName:                   "bbb",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}).Return(nil)

	s.storage.EXPECT().SessionImpersonate(s.testCtx, s.expectUserEmail()).Return(s.expectSessionAuth(0), nil)

	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	role := &storage.Role{
		Attributes:  nil,
		ClientRole:  false,
		Composite:   false,
		Composites:  keycloak.RoleRepresentationCompositesRepresentation{},
		ContainerId: "",
		Description: "",
		Id:          "444",
		Name:        "admin",
	}

	s.storage.EXPECT().RoleGetByCode(s.testCtx, "admin").Return(role, nil)

	s.storage.EXPECT().UserUpdateRoles(s.testCtx, s.expectUserID(), []*storage.Role{role}).Return(nil)

	s.account.EXPECT().Create(s.testCtx, &account.CreateRequest{
		Account: acc,
	}).Return(&account.CreateResponse{
		Account:      acc,
		Success:      false,
		Valid:        true,
		Verification: nil,
	}, nil)

	s.notification.EXPECT().SendNotification(s.testCtx, gomock.Any()).Return(
		&notification.SendNotificationResponse{},
		nil,
	)

	resp, err := s.srv.Signup(s.testCtx, &auth.SignupRequest{
		Account:          acc,
		Password:         &auth.SignupRequest_PasswordValue{PasswordValue: "rrrrrrrr"},
		Pin:              nil,
		PhoneOtp:         &auth.SignupRequest_PhoneOtpValue{PhoneOtpValue: "123456"},
		EmailOtp:         nil,
		TosAccepted:      &auth.SignupRequest_TosAcceptedValue{TosAcceptedValue: 1},
		Remote:           &auth.SignupRequest_RemoteValue{RemoteValue: s.expectUserRemote()},
		OauthId:          nil,
		OauthType:        nil,
		AdminSite:        &auth.SignupRequest_AdminSiteValue{AdminSiteValue: true},
		TransientSession: &auth.SignupRequest_TransientSessionValue{TransientSessionValue: "trans1"},
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Empty(resp.GetErrors())
}

func (s *serviceSuite) TestAuthSignup_error_tos_missing() {
	acc := &account.Account{
		Id:        nil,
		CreatedAt: nil,
		Details: &account.Account_DetailsValue{
			DetailsValue: &account.AccountDetails{
				FirstName: &account.AccountDetails_FirstNameValue{FirstNameValue: "aaa"},
				LastName:  &account.AccountDetails_LastNameValue{LastNameValue: "bbb"},
				Gender:    nil,
				UkNi:      nil,
				Birthday:  nil,
				Avatar:    nil,
			},
		},
		Location: nil,
		Contact: &account.Account_ContactValue{
			ContactValue: &account.AccountContact{
				EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
				PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
				PhonesSecondary: nil,
				ContactStatus:   nil,
			},
		},
		Favorites:           nil,
		Admin:               nil,
		Block:               nil,
		TosVersion:          nil,
		LastLogin:           nil,
		Deleted:             nil,
		Log:                 nil,
		Test:                nil,
		BankDetails:         nil,
		Resources:           nil,
		CreationType:        nil,
		ViewedVideosIds:     nil,
		EnableLandlordTool:  nil,
		Entrypoint:          0,
		AuthorizationOrigin: 0,
	}

	s.account.EXPECT().Verify(s.testCtx, &account.VerifyRequest{
		Account: acc,
	}).Return(&account.VerifyResponse{
		Valid:        true,
		Verification: nil,
	}, nil)

	s.storage.EXPECT().UserGetByEmail(s.testCtx, s.expectUserEmail()).Return(nil, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: "newuser-" + s.expectUserRemote(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
			Key:    "71002003040",
			UserId: "newuser-" + s.expectUserRemote(),
			Remote: "",
			Otp:    "123456",
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: "newuser-" + s.expectUserRemote(),
		Remote: "",
		Otp:    "123456",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	resp, err := s.srv.Signup(s.testCtx, &auth.SignupRequest{
		Account:          acc,
		Password:         &auth.SignupRequest_PasswordValue{PasswordValue: "rrrrrrrr"},
		Pin:              nil,
		PhoneOtp:         &auth.SignupRequest_PhoneOtpValue{PhoneOtpValue: "123456"},
		EmailOtp:         nil,
		TosAccepted:      nil,
		Remote:           &auth.SignupRequest_RemoteValue{RemoteValue: s.expectUserRemote()},
		OauthId:          nil,
		OauthType:        nil,
		AdminSite:        nil,
		TransientSession: &auth.SignupRequest_TransientSessionValue{TransientSessionValue: "trans1"},
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Contains(resp.GetErrors(), auth.AuthErrors_AUTH_ERROR_TOS_REVIEW_REQUIRED)
}

func (s *serviceSuite) TestAuthSignup_error_otp_missing() {
	acc := &account.Account{
		Id:        nil,
		CreatedAt: nil,
		Details: &account.Account_DetailsValue{
			DetailsValue: &account.AccountDetails{
				FirstName: &account.AccountDetails_FirstNameValue{FirstNameValue: "aaa"},
				LastName:  &account.AccountDetails_LastNameValue{LastNameValue: "bbb"},
				Gender:    nil,
				UkNi:      nil,
				Birthday:  nil,
				Avatar:    nil,
			},
		},
		Location: nil,
		Contact: &account.Account_ContactValue{
			ContactValue: &account.AccountContact{
				EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
				PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
				PhonesSecondary: nil,
				ContactStatus:   nil,
			},
		},
		Favorites:           nil,
		Admin:               nil,
		Block:               nil,
		TosVersion:          nil,
		LastLogin:           nil,
		Deleted:             nil,
		Log:                 nil,
		Test:                nil,
		BankDetails:         nil,
		Resources:           nil,
		CreationType:        nil,
		ViewedVideosIds:     nil,
		EnableLandlordTool:  nil,
		Entrypoint:          0,
		AuthorizationOrigin: 0,
	}

	s.account.EXPECT().Verify(s.testCtx, &account.VerifyRequest{
		Account: acc,
	}).Return(&account.VerifyResponse{
		Valid:        true,
		Verification: nil,
	}, nil)

	s.storage.EXPECT().UserGetByEmail(s.testCtx, s.expectUserEmail()).Return(nil, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: "newuser-" + s.expectUserRemote(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
			Key:    "71002003040",
			UserId: "newuser-" + s.expectUserRemote(),
			Remote: "",
			Otp:    "123456",
		},
	}, nil)

	resp, err := s.srv.Signup(s.testCtx, &auth.SignupRequest{
		Account:          acc,
		Password:         &auth.SignupRequest_PasswordValue{PasswordValue: "rrrrrrrr"},
		Pin:              nil,
		PhoneOtp:         nil,
		EmailOtp:         nil,
		TosAccepted:      &auth.SignupRequest_TosAcceptedValue{TosAcceptedValue: 0},
		Remote:           &auth.SignupRequest_RemoteValue{RemoteValue: s.expectUserRemote()},
		OauthId:          nil,
		OauthType:        nil,
		AdminSite:        nil,
		TransientSession: &auth.SignupRequest_TransientSessionValue{TransientSessionValue: "trans1"},
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Contains(resp.GetErrors(), auth.AuthErrors_AUTH_ERROR_PHONE_OTP_REQUIRED)
}

func (s *serviceSuite) TestAuthSignup_error_otp_invalid() {
	acc := &account.Account{
		Id:        nil,
		CreatedAt: nil,
		Details: &account.Account_DetailsValue{
			DetailsValue: &account.AccountDetails{
				FirstName: &account.AccountDetails_FirstNameValue{FirstNameValue: "aaa"},
				LastName:  &account.AccountDetails_LastNameValue{LastNameValue: "bbb"},
				Gender:    nil,
				UkNi:      nil,
				Birthday:  nil,
				Avatar:    nil,
			},
		},
		Location: nil,
		Contact: &account.Account_ContactValue{
			ContactValue: &account.AccountContact{
				EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
				PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
				PhonesSecondary: nil,
				ContactStatus:   nil,
			},
		},
		Favorites:           nil,
		Admin:               nil,
		Block:               nil,
		TosVersion:          nil,
		LastLogin:           nil,
		Deleted:             nil,
		Log:                 nil,
		Test:                nil,
		BankDetails:         nil,
		Resources:           nil,
		CreationType:        nil,
		ViewedVideosIds:     nil,
		EnableLandlordTool:  nil,
		Entrypoint:          0,
		AuthorizationOrigin: 0,
	}

	s.account.EXPECT().Verify(s.testCtx, &account.VerifyRequest{
		Account: acc,
	}).Return(&account.VerifyResponse{
		Valid:        true,
		Verification: nil,
	}, nil)

	s.storage.EXPECT().UserGetByEmail(s.testCtx, s.expectUserEmail()).Return(nil, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: "newuser-" + s.expectUserRemote(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
			Key:    "71002003040",
			UserId: "newuser-" + s.expectUserRemote(),
			Remote: "",
			Otp:    "123456",
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: "newuser-" + s.expectUserRemote(),
		Remote: "",
		Otp:    "qqqq",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_FAIL,
	}, nil)

	resp, err := s.srv.Signup(s.testCtx, &auth.SignupRequest{
		Account:          acc,
		Password:         &auth.SignupRequest_PasswordValue{PasswordValue: "rrrrrrrr"},
		Pin:              nil,
		PhoneOtp:         &auth.SignupRequest_PhoneOtpValue{PhoneOtpValue: "qqqq"},
		EmailOtp:         nil,
		TosAccepted:      &auth.SignupRequest_TosAcceptedValue{TosAcceptedValue: 0},
		Remote:           &auth.SignupRequest_RemoteValue{RemoteValue: s.expectUserRemote()},
		OauthId:          nil,
		OauthType:        nil,
		AdminSite:        nil,
		TransientSession: &auth.SignupRequest_TransientSessionValue{TransientSessionValue: "trans1"},
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Contains(resp.GetErrors(), auth.AuthErrors_AUTH_ERROR_PHONE_OTP_INVALID)
}

func (s *serviceSuite) TestAuthSignup_error_otp_expired() {
	acc := &account.Account{
		Id:        nil,
		CreatedAt: nil,
		Details: &account.Account_DetailsValue{
			DetailsValue: &account.AccountDetails{
				FirstName: &account.AccountDetails_FirstNameValue{FirstNameValue: "aaa"},
				LastName:  &account.AccountDetails_LastNameValue{LastNameValue: "bbb"},
				Gender:    nil,
				UkNi:      nil,
				Birthday:  nil,
				Avatar:    nil,
			},
		},
		Location: nil,
		Contact: &account.Account_ContactValue{
			ContactValue: &account.AccountContact{
				EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
				PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
				PhonesSecondary: nil,
				ContactStatus:   nil,
			},
		},
		Favorites:           nil,
		Admin:               nil,
		Block:               nil,
		TosVersion:          nil,
		LastLogin:           nil,
		Deleted:             nil,
		Log:                 nil,
		Test:                nil,
		BankDetails:         nil,
		Resources:           nil,
		CreationType:        nil,
		ViewedVideosIds:     nil,
		EnableLandlordTool:  nil,
		Entrypoint:          0,
		AuthorizationOrigin: 0,
	}

	s.account.EXPECT().Verify(s.testCtx, &account.VerifyRequest{
		Account: acc,
	}).Return(&account.VerifyResponse{
		Valid:        true,
		Verification: nil,
	}, nil)

	s.storage.EXPECT().UserGetByEmail(s.testCtx, s.expectUserEmail()).Return(nil, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: "newuser-" + s.expectUserRemote(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
			Key:    "71002003040",
			UserId: "newuser-" + s.expectUserRemote(),
			Remote: "",
			Otp:    "123456",
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: "newuser-" + s.expectUserRemote(),
		Remote: "",
		Otp:    "123456",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_EXPIRED,
	}, nil)

	resp, err := s.srv.Signup(s.testCtx, &auth.SignupRequest{
		Account:          acc,
		Password:         &auth.SignupRequest_PasswordValue{PasswordValue: "rrrrrrrr"},
		Pin:              nil,
		PhoneOtp:         &auth.SignupRequest_PhoneOtpValue{PhoneOtpValue: "123456"},
		EmailOtp:         nil,
		TosAccepted:      &auth.SignupRequest_TosAcceptedValue{TosAcceptedValue: 0},
		Remote:           &auth.SignupRequest_RemoteValue{RemoteValue: s.expectUserRemote()},
		OauthId:          nil,
		OauthType:        nil,
		AdminSite:        nil,
		TransientSession: &auth.SignupRequest_TransientSessionValue{TransientSessionValue: "trans1"},
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Contains(resp.GetErrors(), auth.AuthErrors_AUTH_ERROR_PHONE_OTP_EXPIRED)
}

func (s *serviceSuite) TestAuthSignup_error_password_short() {
	acc := &account.Account{
		Id:        nil,
		CreatedAt: nil,
		Details: &account.Account_DetailsValue{
			DetailsValue: &account.AccountDetails{
				FirstName: &account.AccountDetails_FirstNameValue{FirstNameValue: "aaa"},
				LastName:  &account.AccountDetails_LastNameValue{LastNameValue: "bbb"},
				Gender:    nil,
				UkNi:      nil,
				Birthday:  nil,
				Avatar:    nil,
			},
		},
		Location: nil,
		Contact: &account.Account_ContactValue{
			ContactValue: &account.AccountContact{
				EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
				PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
				PhonesSecondary: nil,
				ContactStatus:   nil,
			},
		},
		Favorites:           nil,
		Admin:               nil,
		Block:               nil,
		TosVersion:          nil,
		LastLogin:           nil,
		Deleted:             nil,
		Log:                 nil,
		Test:                nil,
		BankDetails:         nil,
		Resources:           nil,
		CreationType:        nil,
		ViewedVideosIds:     nil,
		EnableLandlordTool:  nil,
		Entrypoint:          0,
		AuthorizationOrigin: 0,
	}

	s.account.EXPECT().Verify(s.testCtx, &account.VerifyRequest{
		Account: acc,
	}).Return(&account.VerifyResponse{
		Valid:        true,
		Verification: nil,
	}, nil)

	s.storage.EXPECT().UserGetByEmail(s.testCtx, s.expectUserEmail()).Return(nil, nil)

	resp, err := s.srv.Signup(s.testCtx, &auth.SignupRequest{
		Account:          acc,
		Password:         &auth.SignupRequest_PasswordValue{PasswordValue: "rr"},
		Pin:              nil,
		PhoneOtp:         &auth.SignupRequest_PhoneOtpValue{PhoneOtpValue: "123456"},
		EmailOtp:         nil,
		TosAccepted:      &auth.SignupRequest_TosAcceptedValue{TosAcceptedValue: 0},
		Remote:           &auth.SignupRequest_RemoteValue{RemoteValue: s.expectUserRemote()},
		OauthId:          nil,
		OauthType:        nil,
		AdminSite:        nil,
		TransientSession: &auth.SignupRequest_TransientSessionValue{TransientSessionValue: "trans1"},
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Contains(resp.GetErrors(), auth.AuthErrors_AUTH_ERROR_PASSWORD_TOO_SHORT)
}

func (s *serviceSuite) TestAuthSignup_error_valid() {
	acc := &account.Account{
		Id:        nil,
		CreatedAt: nil,
		Details: &account.Account_DetailsValue{
			DetailsValue: &account.AccountDetails{
				FirstName: &account.AccountDetails_FirstNameValue{FirstNameValue: "aaa"},
				LastName:  &account.AccountDetails_LastNameValue{LastNameValue: "bbb"},
				Gender:    nil,
				UkNi:      nil,
				Birthday:  nil,
				Avatar:    nil,
			},
		},
		Location: nil,
		Contact: &account.Account_ContactValue{
			ContactValue: &account.AccountContact{
				EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
				PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
				PhonesSecondary: nil,
				ContactStatus:   nil,
			},
		},
		Favorites:           nil,
		Admin:               nil,
		Block:               nil,
		TosVersion:          nil,
		LastLogin:           nil,
		Deleted:             nil,
		Log:                 nil,
		Test:                nil,
		BankDetails:         nil,
		Resources:           nil,
		CreationType:        nil,
		ViewedVideosIds:     nil,
		EnableLandlordTool:  nil,
		Entrypoint:          0,
		AuthorizationOrigin: 0,
	}

	s.account.EXPECT().Verify(s.testCtx, &account.VerifyRequest{
		Account: acc,
	}).Return(&account.VerifyResponse{
		Valid:        true,
		Verification: nil,
	}, nil)

	s.storage.EXPECT().UserGetByEmail(s.testCtx, s.expectUserEmail()).Return(nil, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: "newuser-" + s.expectUserRemote(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
			Key:    "71002003040",
			UserId: "newuser-" + s.expectUserRemote(),
			Remote: "",
			Otp:    "123456",
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: "newuser-" + s.expectUserRemote(),
		Remote: "",
		Otp:    "123456",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	s.storage.EXPECT().UserCreate(s.testCtx, &storage.User{
		Access: map[string]interface{}{
			"manageGroupMembership": true,
			"view":                  true,
			"mapRoles":              true,
			"impersonate":           true,
			"manage":                true,
		},
		Attributes: map[string]interface{}{
			"oauth_type":        "",
			"oauth":             "",
			"email":             s.expectUserEmail(),
			"pin":               "",
			"transient_session": "trans1",
			"phoneNumber":       "71002003040",
		},
		ClientConsents:   nil,
		ClientRoles:      nil,
		CreatedTimestamp: 0,
		Credentials: []keycloak.CredentialRepresentation{
			{
				CreatedDate:    0,
				CredentialData: "",
				Id:             "",
				Priority:       0,
				SecretData:     "",
				Temporary:      false,
				Type:           "password",
				UserLabel:      "",
				Value:          "rrrrrrrr",
			},
		},
		DisableableCredentialTypes: []string{"password"},
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    true,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "aaa",
		Groups:                     nil,
		Id:                         "",
		LastName:                   "bbb",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}).Return(nil)

	s.storage.EXPECT().SessionImpersonate(s.testCtx, s.expectUserEmail()).Return(s.expectSessionAuth(0), nil)

	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	s.account.EXPECT().Create(s.testCtx, &account.CreateRequest{
		Account: acc,
	}).Return(nil, s.expectError())

	_, err := s.srv.Signup(s.testCtx, &auth.SignupRequest{
		Account:          acc,
		Password:         &auth.SignupRequest_PasswordValue{PasswordValue: "rrrrrrrr"},
		Pin:              nil,
		PhoneOtp:         &auth.SignupRequest_PhoneOtpValue{PhoneOtpValue: "123456"},
		EmailOtp:         nil,
		TosAccepted:      &auth.SignupRequest_TosAcceptedValue{TosAcceptedValue: 1},
		Remote:           &auth.SignupRequest_RemoteValue{RemoteValue: s.expectUserRemote()},
		OauthId:          nil,
		OauthType:        nil,
		AdminSite:        nil,
		TransientSession: &auth.SignupRequest_TransientSessionValue{TransientSessionValue: "trans1"},
	})

	s.Require().Error(err)
}
