package test

import (
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/auth/internal/storage"
	invitation "gitlab.mashroom.com/mashroom/backend/invitation/api"
)

func (s *serviceSuite) TestAuthUpdatePin_ok() {
	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	s.account.EXPECT().Get(s.testCtx, &account.GetRequest{
		Id: s.expectUserID(),
	}).Return(&account.GetResponse{
		Account: &account.Account{
			Id:        &account.Account_IdValue{IdValue: s.expectUserID()},
			CreatedAt: nil,
			Details:   nil,
			Location:  nil,
			Contact: &account.Account_ContactValue{
				ContactValue: &account.AccountContact{
					EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
					PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
					PhonesSecondary: nil,
					ContactStatus:   nil,
				},
			},
			Favorites:           nil,
			Admin:               nil,
			Block:               nil,
			TosVersion:          nil,
			LastLogin:           nil,
			Deleted:             nil,
			Log:                 nil,
			Test:                nil,
			BankDetails:         nil,
			Resources:           nil,
			CreationType:        nil,
			ViewedVideosIds:     nil,
			EnableLandlordTool:  nil,
			Entrypoint:          0,
			AuthorizationOrigin: 0,
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: s.expectUserID(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
			Key:    "71002003040",
			UserId: s.expectUserID(),
			Remote: "",
			Otp:    "123456",
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: s.expectUserID(),
		Remote: "",
		Otp:    "123456",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	s.storage.EXPECT().UserGetByID(s.testCtx, s.expectUserID()).Return(&storage.User{
		Access:                     nil,
		Attributes:                 map[string]interface{}{},
		ClientConsents:             nil,
		ClientRoles:                nil,
		CreatedTimestamp:           0,
		Credentials:                nil,
		DisableableCredentialTypes: nil,
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    true,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}, nil)

	s.storage.EXPECT().UserUpdate(s.testCtx, &storage.User{
		Access: nil,
		Attributes: map[string]interface{}{
			"pin": "12345",
		},
		ClientConsents:             nil,
		ClientRoles:                nil,
		CreatedTimestamp:           0,
		Credentials:                nil,
		DisableableCredentialTypes: nil,
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    true,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}).Return(nil)

	s.storage.EXPECT().SessionImpersonate(s.testCtx, s.expectUserID()).Return(s.expectSessionAuth(1), nil)

	resp, err := s.srv.UpdatePin(s.testCtx, &auth.UpdatePinRequest{
		AccessToken: s.expectAccessToken(0),
		Password:    nil,
		Pin:         &auth.UpdatePinRequest_PinValue{PinValue: "12345"},
		PhoneOtp:    &auth.UpdatePinRequest_PhoneOtpValue{PhoneOtpValue: "123456"},
		EmailOtp:    nil,
		NewPin:      nil,
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Empty(resp.GetErrors())
}

func (s *serviceSuite) TestAuthUpdatePin_error_pin() {
	resp, err := s.srv.UpdatePin(s.testCtx, &auth.UpdatePinRequest{
		AccessToken: s.expectAccessToken(0),
		Password:    nil,
		Pin:         &auth.UpdatePinRequest_PinValue{PinValue: "123"},
		PhoneOtp:    &auth.UpdatePinRequest_PhoneOtpValue{PhoneOtpValue: "123456"},
		EmailOtp:    nil,
		NewPin:      nil,
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Contains(resp.GetErrors(), auth.AuthErrors_AUTH_ERROR_PIN_INVALID)
}
