package test

import (
	"github.com/golang/mock/gomock"
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/auth/internal/storage"
	"gitlab.mashroom.com/mashroom/backend/auth/keycloak"
	invitation "gitlab.mashroom.com/mashroom/backend/invitation/api"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
)

func (s *serviceSuite) TestAuthUpdate_ok() {
	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	s.storage.EXPECT().UserGetByID(s.testCtx, s.expectUserID()).Return(&storage.User{
		Access:                     nil,
		Attributes:                 map[string]interface{}{},
		ClientConsents:             nil,
		ClientRoles:                nil,
		CreatedTimestamp:           0,
		Credentials:                nil,
		DisableableCredentialTypes: nil,
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    true,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}, nil)

	acc := &account.Account{
		Id:        &account.Account_IdValue{IdValue: s.expectUserID()},
		CreatedAt: nil,
		Details: &account.Account_DetailsValue{
			DetailsValue: &account.AccountDetails{
				FirstName: &account.AccountDetails_FirstNameValue{FirstNameValue: "aaa"},
				LastName:  &account.AccountDetails_LastNameValue{LastNameValue: "bbb"},
				Gender:    nil,
				UkNi:      nil,
				Birthday:  nil,
				Avatar:    nil,
			},
		},
		Location: nil,
		Contact: &account.Account_ContactValue{
			ContactValue: &account.AccountContact{
				EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
				PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
				PhonesSecondary: nil,
				ContactStatus:   nil,
			},
		},
		Favorites:           nil,
		Admin:               nil,
		Block:               nil,
		TosVersion:          nil,
		LastLogin:           nil,
		Deleted:             nil,
		Log:                 nil,
		Test:                nil,
		BankDetails:         nil,
		Resources:           nil,
		CreationType:        nil,
		ViewedVideosIds:     nil,
		EnableLandlordTool:  nil,
		Entrypoint:          0,
		AuthorizationOrigin: 0,
	}

	s.account.EXPECT().Get(s.testCtx, &account.GetRequest{
		Id:    s.expectUserID(),
		Admin: false,
		Soft:  false,
	}).Return(&account.GetResponse{Account: acc}, nil)

	reqacc := &account.Account{
		Id: &account.Account_IdValue{IdValue: s.expectUserID()},
		Contact: &account.Account_ContactValue{
			ContactValue: &account.AccountContact{
				EmailPrimary: &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: "xxx@xxx.xxx"},
				PhonePrimary: &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003041"},
			},
		},
	}

	s.account.EXPECT().Verify(s.testCtx, &account.VerifyRequest{
		Account: reqacc,
	}).Return(&account.VerifyResponse{
		Valid:        true,
		Verification: nil,
	}, nil)

	s.storage.EXPECT().SessionLogin(s.testCtx, s.expectUserEmail(), "rrr").Return(s.expectSessionAuth(1), nil)
	s.storage.EXPECT().SessionRemove(s.testCtx, s.expectSessionState(1)).Return(nil)

	s.storage.EXPECT().UserGetByEmail(s.testCtx, "xxx@xxx.xxx").Return([]*storage.User{
		{
			Id: s.expectUserID(),
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
		Key:    "xxx@xxx.xxx",
		UserId: s.expectUserID(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
			Key:    "xxx@xxx.xxx",
			UserId: s.expectUserID(),
			Remote: "",
			Otp:    "12345678",
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
		Key:    "xxx@xxx.xxx",
		UserId: s.expectUserID(),
		Remote: "",
		Otp:    "12345678",
		Force:  true,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003041",
		UserId: s.expectUserID(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
			Key:    "71002003041",
			UserId: s.expectUserID(),
			Remote: "",
			Otp:    "123456",
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003041",
		UserId: s.expectUserID(),
		Remote: "",
		Otp:    "123456",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: s.expectUserID(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
			Key:    "71002003040",
			UserId: s.expectUserID(),
			Remote: "",
			Otp:    "123456",
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: s.expectUserID(),
		Remote: "",
		Otp:    "123456",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	s.account.EXPECT().Update(s.testCtx, &account.UpdateRequest{
		Account: reqacc,
	}).Return(&account.UpdateResponse{
		Account: acc,
		Valid:   true,
	}, nil)

	s.storage.EXPECT().UserUpdate(s.testCtx, &storage.User{
		Access: nil,
		Attributes: map[string]interface{}{
			"email":       "xxx@xxx.xxx",
			"phoneNumber": "71002003041",
		},
		ClientConsents:   nil,
		ClientRoles:      nil,
		CreatedTimestamp: 0,
		Credentials: []keycloak.CredentialRepresentation{
			{
				CreatedDate:    0,
				CredentialData: "",
				Id:             "",
				Priority:       0,
				SecretData:     "",
				Temporary:      false,
				Type:           "password",
				UserLabel:      "",
				Value:          "kkkkkkkk",
			},
		},
		DisableableCredentialTypes: nil,
		Email:                      "xxx@xxx.xxx",
		EmailVerified:              false,
		Enabled:                    true,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   "xxx@xxx.xxx",
	}).Return(nil)

	s.notification.EXPECT().SendNotification(s.testCtx, gomock.Any()).Return(
		&notification.SendNotificationResponse{},
		nil,
	)

	s.notification.EXPECT().SendNotification(s.testCtx, gomock.Any()).Return(
		&notification.SendNotificationResponse{},
		nil,
	)

	s.storage.EXPECT().SessionImpersonate(s.testCtx, s.expectUserID()).Return(s.expectSessionAuth(1), nil)

	resp, err := s.srv.Update(s.testCtx, &auth.UpdateRequest{
		AccessToken: s.expectAccessToken(0),
		Account:     reqacc,
		Password:    &auth.UpdateRequest_PasswordValue{PasswordValue: "rrr"},
		Pin:         nil,
		PhoneOtp:    &auth.UpdateRequest_PhoneOtpValue{PhoneOtpValue: "123456"},
		EmailOtp:    &auth.UpdateRequest_EmailOtpValue{EmailOtpValue: "12345678"},
		NewPassword: &auth.UpdateRequest_NewPasswordValue{NewPasswordValue: "kkkkkkkk"},
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().True(resp.GetValid())

	s.Require().Empty(resp.GetErrors())
}

func (s *serviceSuite) TestAuthUpdate_error_invalid() {
	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	s.storage.EXPECT().UserGetByID(s.testCtx, s.expectUserID()).Return(&storage.User{
		Access:                     nil,
		Attributes:                 map[string]interface{}{},
		ClientConsents:             nil,
		ClientRoles:                nil,
		CreatedTimestamp:           0,
		Credentials:                nil,
		DisableableCredentialTypes: nil,
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    true,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}, nil)

	acc := &account.Account{
		Id:        &account.Account_IdValue{IdValue: s.expectUserID()},
		CreatedAt: nil,
		Details: &account.Account_DetailsValue{
			DetailsValue: &account.AccountDetails{
				FirstName: &account.AccountDetails_FirstNameValue{FirstNameValue: "aaa"},
				LastName:  &account.AccountDetails_LastNameValue{LastNameValue: "bbb"},
				Gender:    nil,
				UkNi:      nil,
				Birthday:  nil,
				Avatar:    nil,
			},
		},
		Location: nil,
		Contact: &account.Account_ContactValue{
			ContactValue: &account.AccountContact{
				EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
				PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
				PhonesSecondary: nil,
				ContactStatus:   nil,
			},
		},
		Favorites:           nil,
		Admin:               nil,
		Block:               nil,
		TosVersion:          nil,
		LastLogin:           nil,
		Deleted:             nil,
		Log:                 nil,
		Test:                nil,
		BankDetails:         nil,
		Resources:           nil,
		CreationType:        nil,
		ViewedVideosIds:     nil,
		EnableLandlordTool:  nil,
		Entrypoint:          0,
		AuthorizationOrigin: 0,
	}

	s.account.EXPECT().Get(s.testCtx, &account.GetRequest{
		Id:    s.expectUserID(),
		Admin: false,
		Soft:  false,
	}).Return(&account.GetResponse{Account: acc}, nil)

	reqacc := &account.Account{
		Id: &account.Account_IdValue{IdValue: s.expectUserID()},
		Contact: &account.Account_ContactValue{
			ContactValue: &account.AccountContact{
				EmailPrimary: &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: "xxx@xxx.xxx"},
				PhonePrimary: &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003041"},
			},
		},
	}

	s.account.EXPECT().Verify(s.testCtx, &account.VerifyRequest{
		Account: reqacc,
	}).Return(&account.VerifyResponse{
		Valid:        true,
		Verification: nil,
	}, nil)

	s.storage.EXPECT().SessionLogin(s.testCtx, s.expectUserEmail(), "rrr").Return(s.expectSessionAuth(1), nil)
	s.storage.EXPECT().SessionRemove(s.testCtx, s.expectSessionState(1)).Return(nil)

	s.storage.EXPECT().UserGetByEmail(s.testCtx, "xxx@xxx.xxx").Return([]*storage.User{
		{
			Id: s.expectUserID(),
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
		Key:    "xxx@xxx.xxx",
		UserId: s.expectUserID(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
			Key:    "xxx@xxx.xxx",
			UserId: s.expectUserID(),
			Remote: "",
			Otp:    "12345678",
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
		Key:    "xxx@xxx.xxx",
		UserId: s.expectUserID(),
		Remote: "",
		Otp:    "12345678",
		Force:  true,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003041",
		UserId: s.expectUserID(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
			Key:    "71002003041",
			UserId: s.expectUserID(),
			Remote: "",
			Otp:    "123456",
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003041",
		UserId: s.expectUserID(),
		Remote: "",
		Otp:    "123456",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: s.expectUserID(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
			Key:    "71002003040",
			UserId: s.expectUserID(),
			Remote: "",
			Otp:    "123456",
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: s.expectUserID(),
		Remote: "",
		Otp:    "123456",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	s.account.EXPECT().Update(s.testCtx, &account.UpdateRequest{
		Account: reqacc,
	}).Return(&account.UpdateResponse{
		Account: acc,
		Valid:   false,
	}, nil)

	resp, err := s.srv.Update(s.testCtx, &auth.UpdateRequest{
		AccessToken: s.expectAccessToken(0),
		Account:     reqacc,
		Password:    &auth.UpdateRequest_PasswordValue{PasswordValue: "rrr"},
		Pin:         nil,
		PhoneOtp:    &auth.UpdateRequest_PhoneOtpValue{PhoneOtpValue: "123456"},
		EmailOtp:    &auth.UpdateRequest_EmailOtpValue{EmailOtpValue: "12345678"},
		NewPassword: &auth.UpdateRequest_NewPasswordValue{NewPasswordValue: "kkkkkkkk"},
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().False(resp.GetValid())

	s.Require().Empty(resp.GetErrors())
}

func (s *serviceSuite) TestAuthUpdate_error_update() {
	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	s.storage.EXPECT().UserGetByID(s.testCtx, s.expectUserID()).Return(&storage.User{
		Access:                     nil,
		Attributes:                 map[string]interface{}{},
		ClientConsents:             nil,
		ClientRoles:                nil,
		CreatedTimestamp:           0,
		Credentials:                nil,
		DisableableCredentialTypes: nil,
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    true,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}, nil)

	acc := &account.Account{
		Id:        &account.Account_IdValue{IdValue: s.expectUserID()},
		CreatedAt: nil,
		Details: &account.Account_DetailsValue{
			DetailsValue: &account.AccountDetails{
				FirstName: &account.AccountDetails_FirstNameValue{FirstNameValue: "aaa"},
				LastName:  &account.AccountDetails_LastNameValue{LastNameValue: "bbb"},
				Gender:    nil,
				UkNi:      nil,
				Birthday:  nil,
				Avatar:    nil,
			},
		},
		Location: nil,
		Contact: &account.Account_ContactValue{
			ContactValue: &account.AccountContact{
				EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
				PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
				PhonesSecondary: nil,
				ContactStatus:   nil,
			},
		},
		Favorites:           nil,
		Admin:               nil,
		Block:               nil,
		TosVersion:          nil,
		LastLogin:           nil,
		Deleted:             nil,
		Log:                 nil,
		Test:                nil,
		BankDetails:         nil,
		Resources:           nil,
		CreationType:        nil,
		ViewedVideosIds:     nil,
		EnableLandlordTool:  nil,
		Entrypoint:          0,
		AuthorizationOrigin: 0,
	}

	s.account.EXPECT().Get(s.testCtx, &account.GetRequest{
		Id:    s.expectUserID(),
		Admin: false,
		Soft:  false,
	}).Return(&account.GetResponse{Account: acc}, nil)

	reqacc := &account.Account{
		Id: &account.Account_IdValue{IdValue: s.expectUserID()},
		Contact: &account.Account_ContactValue{
			ContactValue: &account.AccountContact{
				EmailPrimary: &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: "xxx@xxx.xxx"},
				PhonePrimary: &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003041"},
			},
		},
	}

	s.account.EXPECT().Verify(s.testCtx, &account.VerifyRequest{
		Account: reqacc,
	}).Return(&account.VerifyResponse{
		Valid:        true,
		Verification: nil,
	}, nil)

	s.storage.EXPECT().SessionLogin(s.testCtx, s.expectUserEmail(), "rrr").Return(s.expectSessionAuth(1), nil)
	s.storage.EXPECT().SessionRemove(s.testCtx, s.expectSessionState(1)).Return(nil)

	s.storage.EXPECT().UserGetByEmail(s.testCtx, "xxx@xxx.xxx").Return([]*storage.User{
		{
			Id: s.expectUserID(),
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
		Key:    "xxx@xxx.xxx",
		UserId: s.expectUserID(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
			Key:    "xxx@xxx.xxx",
			UserId: s.expectUserID(),
			Remote: "",
			Otp:    "12345678",
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
		Key:    "xxx@xxx.xxx",
		UserId: s.expectUserID(),
		Remote: "",
		Otp:    "12345678",
		Force:  true,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003041",
		UserId: s.expectUserID(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
			Key:    "71002003041",
			UserId: s.expectUserID(),
			Remote: "",
			Otp:    "123456",
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003041",
		UserId: s.expectUserID(),
		Remote: "",
		Otp:    "123456",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: s.expectUserID(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
			Key:    "71002003040",
			UserId: s.expectUserID(),
			Remote: "",
			Otp:    "123456",
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_PHONE,
		Key:    "71002003040",
		UserId: s.expectUserID(),
		Remote: "",
		Otp:    "123456",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	s.account.EXPECT().Update(s.testCtx, &account.UpdateRequest{
		Account: reqacc,
	}).Return(&account.UpdateResponse{
		Account: acc,
		Valid:   true,
	}, nil)

	s.storage.EXPECT().UserUpdate(s.testCtx, &storage.User{
		Access: nil,
		Attributes: map[string]interface{}{
			"email":       "xxx@xxx.xxx",
			"phoneNumber": "71002003041",
		},
		ClientConsents:   nil,
		ClientRoles:      nil,
		CreatedTimestamp: 0,
		Credentials: []keycloak.CredentialRepresentation{
			{
				CreatedDate:    0,
				CredentialData: "",
				Id:             "",
				Priority:       0,
				SecretData:     "",
				Temporary:      false,
				Type:           "password",
				UserLabel:      "",
				Value:          "kkkkkkkk",
			},
		},
		DisableableCredentialTypes: nil,
		Email:                      "xxx@xxx.xxx",
		EmailVerified:              false,
		Enabled:                    true,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   "xxx@xxx.xxx",
	}).Return(s.expectError())

	_, err := s.srv.Update(s.testCtx, &auth.UpdateRequest{
		AccessToken: s.expectAccessToken(0),
		Account:     reqacc,
		Password:    &auth.UpdateRequest_PasswordValue{PasswordValue: "rrr"},
		Pin:         nil,
		PhoneOtp:    &auth.UpdateRequest_PhoneOtpValue{PhoneOtpValue: "123456"},
		EmailOtp:    &auth.UpdateRequest_EmailOtpValue{EmailOtpValue: "12345678"},
		NewPassword: &auth.UpdateRequest_NewPasswordValue{NewPasswordValue: "kkkkkkkk"},
	})

	s.Require().Error(err)
}
