//go:generate mockgen -package=mockstorage -destination=mock/repository.go . Repository

// Package storage  provides structures for user persistent repository
package storage

import "context"

// Repository provides user/session repository abstraction
type Repository interface {
	Migrate(ctx context.Context) error
	SessionRefresh(ctx context.Context, refreshToken string) (*SessionAuth, error)
	SessionLogin(ctx context.Context, user, password string) (*SessionAuth, error)
	SessionIntrospect(ctx context.Context, token string) (*SessionIntrospection, error)
	SessionImpersonate(ctx context.Context, userID string) (*SessionAuth, error)
	SessionRemove(ctx context.Context, sessionID string) error
	SessionRemoveAll(ctx context.Context, userID string) error
	UserCreate(ctx context.Context, user *User) error
	UserUpdate(ctx context.Context, user *User) error
	UserUpdateRoles(ctx context.Context, userID string, roles []*Role) error
	UserPasswordReset(ctx context.Context, userID, password string) error
	UserGetByID(ctx context.Context, userID string) (*User, error)
	UserGetByEmail(ctx context.Context, email string) ([]*User, error)
	RoleGetByCode(ctx context.Context, code string) (*Role, error)
}
